@@ -30,6 +30,13 @@ from .sched_async import AsyncScheduler
 if True:  # ASYNC
     import asyncio
 
+    # The exceptions that we need to capture in order to keep the pool
+    # consistent and avoid losing connections on errors in callers code.
+    CLIENT_EXCEPTIONS = (Exception, asyncio.CancelledError)
+else:
+    CLIENT_EXCEPTIONS = Exception
+
+
 logger = logging.getLogger("psycopg.pool")
 
 
@@ -237,7 +244,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
             conn = await self._getconn_unchecked(deadline - monotonic())
             try:
                 await self._check_connection(conn)
-            except Exception:
+            except CLIENT_EXCEPTIONS:
                 await self._putconn(conn, from_getconn=True)
             else:
                 logger.info("connection given by %r", self.name)
@@ -275,7 +282,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
         if not conn:
             try:
                 conn = await pos.wait(timeout=timeout)
-            except Exception:
+            except CLIENT_EXCEPTIONS:
                 self._stats[self._REQUESTS_ERRORS] += 1
                 raise
             finally:
@@ -312,7 +319,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
             return
         try:
             await self._check(conn)
-        except Exception as e:
+        except CLIENT_EXCEPTIONS as e:
             logger.info("connection failed check: %s", e)
             raise
 
@@ -358,8 +365,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
         if not self._closed:
             return False
 
-        conn._pool = None
-        await conn.close()
+        await self._close_connection(conn)
         return True
 
     async def open(self, wait: bool = False, timeout: float = 30.0) -> None:
@@ -473,7 +479,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
 
         # Close the connections that were still in the pool
         for conn in connections:
-            await conn.close()
+            await self._close_connection(conn)
 
         # Signal to eventual clients in the queue that business is closed.
         for pos in waiting:
@@ -545,14 +551,14 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
             # Check for expired connections
             if conn._expire_at <= monotonic():
                 logger.info("discarding expired connection %s", conn)
-                await conn.close()
+                await self._close_connection(conn)
                 self.run_task(AddConnection(self))
                 continue
 
             # Check for broken connections
             try:
                 await self.check_connection(conn)
-            except Exception:
+            except CLIENT_EXCEPTIONS:
                 self._stats[self._CONNECTIONS_LOST] += 1
                 logger.warning("discarding broken connection: %s", conn)
                 self.run_task(AddConnection(self))
@@ -631,7 +637,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
             # Run the task. Make sure don't die in the attempt.
             try:
                 await task.run()
-            except Exception as ex:
+            except CLIENT_EXCEPTIONS as ex:
                 logger.warning(
                     "task run %s failed: %s: %s", task, ex.__class__.__name__, ex
                 )
@@ -646,7 +652,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
         t0 = monotonic()
         try:
             conn = await self.connection_class.connect(self.conninfo, **kwargs)
-        except Exception:
+        except CLIENT_EXCEPTIONS:
             self._stats[self._CONNECTIONS_ERRORS] += 1
             raise
         else:
@@ -684,8 +690,8 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
 
         try:
             conn = await self._connect()
-        except Exception as ex:
-            logger.warning(f"error connecting in {self.name!r}: {ex}")
+        except CLIENT_EXCEPTIONS as ex:
+            logger.warning("error connecting in %r: %s", self.name, ex)
             if attempt.time_to_give_up(now):
                 logger.warning(
                     "reconnection attempt in pool %r failed after %s sec",
@@ -724,28 +730,28 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
         """
         Return a connection to the pool after usage.
         """
+        await self._reset_connection(conn)
         if from_getconn:
             if conn.pgconn.transaction_status == TransactionStatus.UNKNOWN:
                 self._stats[self._CONNECTIONS_LOST] += 1
                 # Connection no more in working state: create a new one.
-                self.run_task(AddConnection(self))
                 logger.info("not serving connection found broken")
+                self.run_task(AddConnection(self))
                 return
 
         else:
-            await self._reset_connection(conn)
             if conn.pgconn.transaction_status == TransactionStatus.UNKNOWN:
                 self._stats[self._RETURNS_BAD] += 1
                 # Connection no more in working state: create a new one.
-                self.run_task(AddConnection(self))
                 logger.warning("discarding closed connection: %s", conn)
+                self.run_task(AddConnection(self))
                 return
 
         # Check if the connection is past its best before date
         if conn._expire_at <= monotonic():
-            self.run_task(AddConnection(self))
             logger.info("discarding expired connection")
-            await conn.close()
+            await self._close_connection(conn)
+            self.run_task(AddConnection(self))
             return
 
         await self._add_to_pool(conn)
@@ -769,7 +775,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
         # between here and entering the lock. Therefore we will make another
         # check later.
         if self._closed:
-            await conn.close()
+            await self._close_connection(conn)
             return
 
         # Critical section: if there is a client waiting give it the connection
@@ -781,7 +787,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
             # this connection while the main process is closing the pool.
             # Now that we are in the critical section we know for real.
             if self._closed:
-                await conn.close()
+                await self._close_connection(conn)
                 return
 
             while self._waiting:
@@ -813,19 +819,19 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
             logger.warning("rolling back returned connection: %s", conn)
             try:
                 await conn.rollback()
-            except Exception as ex:
+            except CLIENT_EXCEPTIONS as ex:
                 logger.warning(
                     "rollback failed: %s: %s. Discarding connection %s",
                     ex.__class__.__name__,
                     ex,
                     conn,
                 )
-                await conn.close()
+                await self._close_connection(conn)
 
         elif status == TransactionStatus.ACTIVE:
             # Connection returned during an operation. Bad... just close it.
             logger.warning("closing returned connection: %s", conn)
-            await conn.close()
+            await self._close_connection(conn)
 
         if self._reset:
             try:
@@ -836,9 +842,13 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
                         f"connection left in status {sname} by reset function"
                         f" {self._reset}: discarded"
                     )
-            except Exception as ex:
-                logger.warning(f"error resetting connection: {ex}")
-                await conn.close()
+            except CLIENT_EXCEPTIONS as ex:
+                logger.warning("error resetting connection: %s", ex)
+                await self._close_connection(conn)
+
+    async def _close_connection(self, conn: ACT) -> None:
+        conn._pool = None
+        await conn.close()
 
     async def _shrink_pool(self) -> None:
         to_close: ACT | None = None
@@ -849,7 +859,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
             self._nconns_min = len(self._pool)
 
             # If the pool can shrink and connections were unused, drop one
-            if self._nconns > self._min_size and nconns_min > 0:
+            if self._nconns > self._min_size and nconns_min > 0 and self._pool:
                 to_close = self._pool.popleft()
                 self._nconns -= 1
                 self._nconns_min -= 1
@@ -863,7 +873,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
                 nconns_min,
                 self.max_idle,
             )
-            await to_close.close()
+            await self._close_connection(to_close)
 
     def _get_measures(self) -> dict[str, int]:
         rv = super()._get_measures()
@@ -899,7 +909,7 @@ class WaitingClient(Generic[ACT]):
                         self.error = PoolTimeout(
                             f"couldn't get a connection after {timeout:.2f} sec"
                         )
-                except BaseException as ex:
+                except CLIENT_EXCEPTIONS as ex:
                     self.error = ex
 
         if self.conn:
