@@ -1020,30 +1020,56 @@ async def test_check_backoff(dsn, caplog, monkeypatch):
         want *= 2
 
 
+@pytest.mark.slow
+@pytest.mark.parametrize("status", ["ERROR", "INTRANS"])
+async def test_check_returns_an_ok_connection(dsn, status):
+    async def check(conn):
+        if status == "ERROR":
+            await conn.execute("wat")
+        elif status == "INTRANS":
+            await conn.execute("select 1")
+            1 / 0
+        else:
+            assert False
+
+    async with pool.AsyncConnectionPool(dsn, min_size=1, check=check) as p:
+        await p.wait(1.0)
+        with pytest.raises(pool.PoolTimeout):
+            conn = await p.getconn(0.5)
+
+        conn = list(p._pool)[0]
+        assert conn.info.transaction_status == TransactionStatus.IDLE
+
+
+class ReturningConnection(psycopg.AsyncConnection[Row]):
+    """
+    Test connection returning to the pool on close.
+
+    Verify that it's possible to override `close()` to act as `putconn()`.
+    which allows to use the psycopg pool in a sqlalchemy NullPool.
+
+    We cannot guarantee 100% that we will never break this implementation,
+    but we can keep awareness that we use it this way, maintain it on a
+    best-effort basis, and notify upstream if we are forced to break it.
+
+    https://github.com/sqlalchemy/sqlalchemy/discussions/12522
+    https://github.com/psycopg/psycopg/issues/1046
+    """
+
+    async def close(self) -> None:
+        if pool := getattr(self, "_pool", None):
+            # Connection currently checked out from the pool.
+            # Instead of closing it, return it to the pool.
+            await pool.putconn(self)
+        else:
+            # Connection not part of any pool, or currently into the pool.
+            # Close the connection for real.
+            await super().close()
+
+
 async def test_override_close(dsn):
-    # Verify that it's possible to override `close()` to act as `putconn()`.
-    # which allows to use the psycopg pool in a sqlalchemy NullPool.
-    #
-    # We cannot guarantee 100% that we will never break this implementation,
-    # but we can keep awareness that we use it this way, maintain it on a
-    # best-effort basis, and notify upstream if we are forced to break it.
-    #
-    # https://github.com/sqlalchemy/sqlalchemy/discussions/12522
-    # https://github.com/psycopg/psycopg/issues/1046
-
-    class MyConnection(psycopg.AsyncConnection[Row]):
-        async def close(self) -> None:
-            if pool := getattr(self, "_pool", None):
-                # Connection currently checked out from the pool.
-                # Instead of closing it, return it to the pool.
-                await pool.putconn(self)
-            else:
-                # Connection not part of any pool, or currently into the pool.
-                # Close the connection for real.
-                await super().close()
-
     async with pool.AsyncConnectionPool(
-        dsn, connection_class=MyConnection, min_size=2
+        dsn, connection_class=ReturningConnection, min_size=2
     ) as p:
         await p.wait()
         assert len(p._pool) == 2
@@ -1055,3 +1081,23 @@ async def test_override_close(dsn):
         assert len(p._pool) == 2
 
     assert conn.closed
+
+
+@pytest.mark.slow
+async def test_override_close_no_loop(dsn):
+    async with pool.AsyncConnectionPool(
+        dsn, min_size=1, max_lifetime=0.05, connection_class=ReturningConnection
+    ) as p:
+        conn = await p.getconn()
+        await asleep(0.1)
+        assert len(p._pool) == 0
+        await asleep(0.1)  # wait for the connection to expire
+        await conn.close()
+        await asleep(0.1)
+        assert len(p._pool) == 1
+        conn = await p.getconn()
+        await asleep(0.1)
+        assert len(p._pool) == 0
+        await conn.close()
+        await asleep(0.1)
+        assert len(p._pool) == 1
